; ===================================================================
; Project: HistoryCombo Library
; Description: Implements a ComboBox control that supports keyboard events for adding to and
;	removing entries from the list.
; Author: Jason Boggs <vampire DOT valik AT gmail DOT com>
; ===================================================================
#include-once

#Region Members Exported
#cs
_HistoryComboCreate($hParent, $x, $y, $w = Default, $h = Default) - Creates a new instance of the HistoryCombo control.
_HistoryComboDestroy(ByRef $aCombo) - Destroys a HistorCombo control.
_HistoryComboSetPosition($aCombo, $x, $y, $w, $h) - Sets the position of the control.
_HistoryComboSetSelected(ByRef $vCombo, $vSelect) - Sets the selected item in the control.
_HistoryComboGetSelected(ByRef $vCombo) - Retrieves the text of the selected item.
_HistoryComboAddItem(ByRef $vCombo, $sText, $bUnique = True) - Adds a new item to the list.
_HistoryComboRemoveItem(ByRef $vCombo, $nIndex) - Removes an item from the list based on index.
_HistoryComboAddList(ByRef $vCombo, $sList, $sDelimiter = "|") - Adds a delimited list to the control.
_HistoryComboGetList(ByRef $vCombo, $sDelimiter = "|") - Retrieves a delimited list of items in the control.
_HistoryComboGetObj(ByRef $vCombo) - Retrieves the COM object for the ComboBox control.
_HistoryComboGetControlID(ByRef $aCombo) - Retrieves the controld ID of the ActiveX control.
_HistoryComboGetEventObj(ByRef $aCombo) - Retrieves the COM event sink object.
_HistoryComboShowDropdown(ByRef $vCombo, $bShow = True) - Displays or hides the dropdown list.
_HistoryComboIsDropped(ByRef $vCombo) - Retrieve the visibility state of the dropdown list.
#ce
#EndRegion Members Exported

#Region Includes
#EndRegion Includes

#Region Global Variables
Global $__g_HistoryComboItem	; Holds the currently active object when Ctrl is pressed
Global Const $HISTORYCOMBO_MAX = 3	; Maximum array size
#EndRegion Global Variables

#Region Public Members

#Region _HistoryComboCreate()
; ===================================================================
; _HistoryComboCreate($hParent, $x, $y, $w = Default, $h = Default)
;
; Creates a new instance of the HistoryCombo control.
; Parameters:
;	$hParent - IN - Handle to the parent window for the control.
;	$x - IN - Left position of the control.
;	$y - IN - Top position of the control.
;	$w - IN/OPTIONAL - Width of the control.
;	$h - IN/OPTIONAL - Height of the control.
; Returns:
;	Success - An opaque handle to the control.
;	Failure - Sets @error and returns 0 if the control could not be created.
; ===================================================================
Func _HistoryComboCreate($hParent, $x, $y, $w = Default, $h = Default)
	Local $oCombo = ObjCreate("MSComctlLib.ImageComboCtl")
	If @error Then
		SetError(1)
		Return 0
	EndIf
	$oCombo.Enabled = True	; Must enable the control
	$oCombo.MousePointer = 0	; Sometimes the control uses non-default so force default
	GUISwitch($hParent)	; We want to make sure we create on the proper parent
	; If we don't create the control really huge and then size it back down, we can't change the size at all!
	Local $idObj = GUICtrlCreateObj($oCombo, 0, 0, 10000, 10000)
	; Now we can resize to what the user wants
	GUICtrlSetPos($idObj, $x, $y, $w, $h)
	; We have to do this so the control will show up on tabbed GUIs
	Local Const $WS_VISIBLE = 0x10000000
	GUICtrlSetStyle($idObj, $WS_VISIBLE)

	Local $oEvent = ObjEvent($oCombo, "_HistoryComboEvent_", "DImageComboEvents")
	Local $aRet[$HISTORYCOMBO_MAX] = [ $oCombo, $idObj, $oEvent ]
	Return $aRet
EndFunc	; _HistoryComboCreate()
#EndRegion _HistoryComboCreate()

#Region _HistoryComboDestroy()
; ===================================================================
; _HistoryComboDestroy(ByRef $aCombo)
;
; Destroys a HistorCombo control.
; Parameters:
;	$aCombo - IN - Handle to the control.
; Returns:
;	True if the control was destroyed, false otherwise.
; ===================================================================
Func _HistoryComboDestroy(ByRef $aCombo)
	If UBound($aCombo) = $HISTORYCOMBO_MAX Then
		$aCombo[2] = 0
		GUICtrlDelete($aCombo[1])
		$aCombo[0] = 0
		Return True
	Else
		Return False
	EndIf
EndFunc	; _HistoryComboDestroy()
#EndRegion _HistoryComboDestroy()

#Region _HistoryComboSetPosition()
; ===================================================================
; _HistoryComboSetPosition($aCombo, $x, $y, $w, $h)
;
; Sets the position of the control.
; Parameters:
;	$aCombo - IN - Handle to the control.
;	$x - IN - Left position of the control.
;	$y - IN - Top position of the control.
;	$w - IN - Width of the control.
;	$h - IN - Height of the control.
; Returns:
;	None.
; ===================================================================
Func _HistoryComboSetPosition($aCombo, $x, $y, $w, $h)
	If UBound($aCombo) = $HISTORYCOMBO_MAX Then
		GUICtrlSetPos(_HistoryComboGetControlID($aCombo), $x, $y, $w, $h)
	EndIf
EndFunc	; _HistoryComboSetPosition()
#EndRegion _HistoryComboSetPosition()

#Region _HistoryComboSetSelected()
; ===================================================================
; _HistoryComboSetSelected(ByRef $vCombo, $vSelect)
;
; Sets the selected item in the control.
; Parameters:
;	$vCombo - IN - Handle to the control.
;	$vSelect - IN - Either the text string to select or the index of an item to select.  The string will
;		not be added if it does not exist.  Empy string/numeric 0 set the selected item to empty.
; Returns:
;	True if a selection was made, False otherwise.
; ===================================================================
Func _HistoryComboSetSelected(ByRef $vCombo, $vSelect)
	Local $oCombo = _HistoryComboGetObj($vCombo)
	If IsObj($oCombo) Then
		If (IsString($vSelect) And $vSelect = "") Or (IsNumber($vSelect) And $vSelect = 0) Then
			$oCombo.Text = ""
			Return True
		Else
			For $oItem In $oCombo.ComboItems
				If Not IsObj($oItem) Then ExitLoop
				If (IsNumber($vSelect) And $oItem.Index = $vSelect) Or _
					(IsString($vSelect) And $oItem.Text = $vSelect) Then
					$oCombo.SelectedItem = $oItem
					Return True
				EndIf
			Next
		EndIf
	EndIf
	Return False
EndFunc	; _HistoryComboSetSelected()
#EndRegion _HistoryComboSetSelected()

#Region _HistoryComboGetSelected()
; ===================================================================
; _HistoryComboGetSelected(ByRef $vCombo)
;
; Retrieves the text of the selected item.
; Parameters:
;	$vCombo - IN - Handle to the control.
; Returns:
;	Text of the selected item or an empty string.
; ===================================================================
Func _HistoryComboGetSelected(ByRef $vCombo)
	Local $oCombo = _HistoryComboGetObj($vCombo)
	If IsObj($oCombo) Then
		Local $oItem = $oCombo.SelectedItem
		If IsObj($oItem) Then Return $oItem.Text
	EndIf
	Return ""
EndFunc	; _HistoryComboGetSelected()
#EndRegion _HistoryComboGetSelected()

#Region _HistoryComboAddItem()
; ===================================================================
; _HistoryComboAddItem(ByRef $vCombo, $sText, $bUnique = True)
;
; Adds a new item to the list.
; Parameters:
;	$vCombo - IN - Handle to the control.
;	$sText - IN - Text to add.
;	$bUnique - IN/OPTIONAL - If true, duplicates will not be added.
; Returns:
;	True if the item was added, False otherwise.
; ===================================================================
Func _HistoryComboAddItem(ByRef $vCombo, $sText, $bUnique = True)
	Local $oCombo = _HistoryComboGetObj($vCombo)
	If IsObj($oCombo) And $sText Then
		If $bUnique Then
			For $oItem In $oCombo.ComboItems
				If Not IsObj($oItem) Then ExitLoop
				If $oItem.Text = $sText Then Return False
			Next
		EndIf
		$oCombo.SelectedItem = $oCombo.ComboItems.Add($oCombo.ComboItems.Count + 1, "", $sText)
		Return True
	Else
		Return False
	EndIf
EndFunc	; _HistoryComboAddItem()
#EndRegion _HistoryComboAddItem()

#Region _HistoryComboRemoveItem()
; ===================================================================
; _HistoryComboRemoveItem(ByRef $vCombo, $nIndex)
;
; Removes an item from the list based on index.
; Parameters:
;	$vCombo - IN - Handle to the control.
;	$nIndex - IN - Index of the item to remove.
; Returns:
;	True if the item was removed, False otherwise.
; ===================================================================
Func _HistoryComboRemoveItem(ByRef $vCombo, $nIndex)
	Local $oCombo = _HistoryComboGetObj($vCombo)
	If IsObj($oCombo) Then
		For $oItem In $oCombo.ComboItems
			If Not IsObj($oItem) Then ExitLoop
			If $oItem.Index= $nIndex Then
				$oCombo.ComboItems.Remove($nIndex)
				Return True
			EndIf
		Next
	EndIf
	Return False
EndFunc	; _HistoryComboRemoveItem()
#EndRegion _HistoryComboRemoveItem()

#Region _HistoryComboAddList()
; ===================================================================
; _HistoryComboAddList(ByRef $vCombo, $sList, $sDelimiter = "|")
;
; Adds a delimited list to the control.
; Parameters:
;	$vCombo - IN - Handle to the control.
;	$sList - IN - A delimited string listing all the items to add.
;	$sDelimiter - IN/OPTIONAL - The delimiter to split items by.
; Returns:
;	True if items were added to the list, False otherwise.
; ===================================================================
Func _HistoryComboAddList(ByRef $vCombo, $sList, $sDelimiter = "|")
	Local $oCombo = _HistoryComboGetObj($vCombo)
	If IsObj($oCombo) Then
		Local $aList = StringSplit($sList, $sDelimiter)
		For $i = 1 To UBound($aList) - 1
			_HistoryComboAddItem($oCombo, $aList[$i])
		Next
		Return True
	Else
		Return False
	EndIf
EndFunc	; _HistoryComboAddList()
#EndRegion _HistoryComboAddList()

#Region _HistoryComboGetList()
; ===================================================================
; _HistoryComboGetList(ByRef $vCombo, $sDelimiter = "|")
;
; Retrieves a delimited list of items in the control.
; Parameters:
;	$vCombo - IN - Handle to the control.
;	$sDelimiter - IN/OPTIONAL - Delimiter to place between items.
; Returns:
;	A delimited string of items or an empty string.
; ===================================================================
Func _HistoryComboGetList(ByRef $vCombo, $sDelimiter = "|")
	Local $oCombo = _HistoryComboGetObj($vCombo)
	If IsObj($oCombo) Then
		Local $sResult
		For $oItem In $oCombo.ComboItems
			If Not IsObj($oItem) Then ExitLoop
			$sResult &= $oItem.Text & "|"
		Next
		; Only trim the delimiter if the delimiter is found
		If StringRight($sResult, StringLen($sDelimiter)) = $sDelimiter Then
			$sResult = StringTrimRight($sResult, StringLen($sDelimiter))
		EndIf
		Return $sResult
	EndIf
	Return ""
EndFunc	; _HistoryComboGetList()
#EndRegion _HistoryComboGetList()

#Region _HistoryComboGetObj()
; ===================================================================
; _HistoryComboGetObj(ByRef $vCombo)
;
; Retrieves the COM object for the ComboBox control.
; Parameters:
;	$vCombo - IN - Handle to the control.
; Returns:
;	COM object for the control if successful, 0 otherwise.
; ===================================================================
Func _HistoryComboGetObj(ByRef $vCombo)
	If UBound($vCombo) = $HISTORYCOMBO_MAX Then
		Return $vCombo[0]
	ElseIf IsObj($vCombo) Then
		Return $vCombo
	Else
		Return 0
	EndIf
EndFunc	; _HistoryComboGetObj()
#EndRegion _HistoryComboGetObj()

#Region _HistoryComboGetControlID()
; ===================================================================
; _HistoryComboGetControlID(ByRef $aCombo)
;
; Retrieves the controld ID of the ActiveX control.
; Parameters:
;	$aCombo - IN - Handle to the control.
; Returns:
;	Controld ID to the ActiveX control if successful, 0 otherwise.
; ===================================================================
Func _HistoryComboGetControlID(ByRef $aCombo)
	If UBound($aCombo) = $HISTORYCOMBO_MAX Then
		Return $aCombo[1]
	Else
		Return 0
	EndIf
EndFunc	; _HistoryComboGetControlID()
#EndRegion _HistoryComboGetControlID()

#Region _HistoryComboGetEventObj()
; ===================================================================
; _HistoryComboGetEventObj(ByRef $aCombo)
;
; Retrieves the COM event sink object.
; Parameters:
;	$aCombo - IN - Handle to the control.
; Returns:
;	COM event sink object if successful, 0 otherwise.
; ===================================================================
Func _HistoryComboGetEventObj(ByRef $aCombo)
	If UBound($aCombo) = $HISTORYCOMBO_MAX Then
		Return $aCombo[2]
	Else
		Return 0
	EndIf
EndFunc	; _HistoryComboGetEventObj()
#EndRegion _HistoryComboGetEventObj()

#Region _HistoryComboShowDropdown()
; ===================================================================
; _HistoryComboShowDropdown(ByRef $vCombo, $bShow = True)
;
; Displays or hides the dropdown list.
; Parameters:
;	$vCombo - IN - Handle to the control.
;	$bShow - IN/OPTIONAL - True shows the list, False hides it.
; Returns:
;	None.
; ===================================================================
Func _HistoryComboShowDropdown(ByRef $vCombo, $bShow = True)
	Local Const $CB_SHOWDROPDOWN = 0x014F
	__HistoryComboSendMessage($vCombo, $CB_SHOWDROPDOWN, $bShow)
EndFunc	; _HistoryComboShowDropdown()
#EndRegion _HistoryComboShowDropdown()

#Region _HistoryComboIsDropped()
; ===================================================================
; _HistoryComboIsDropped(ByRef $vCombo)
;
; Retrieve the visibility state of the dropdown list.
; Parameters:
;	$vCombo - IN - Handle to the control.
; Returns:
;	True if the dropdown is visible, False otherwise.
; ===================================================================
Func _HistoryComboIsDropped(ByRef $vCombo)
	Local Const $CB_GETDROPPEDSTATE = 0x0157
	Return __HistoryComboSendMessage($vCombo, $CB_GETDROPPEDSTATE)
EndFunc	; _HistoryComboIsDropped()
#EndRegion _HistoryComboIsDropped()

#EndRegion Public Members

#Region COM Event Callbacks

#Region _HistoryComboEvent_KeyDown()
; ===================================================================
; _HistoryComboEvent_KeyDown($nKey, $nShift)
;
; Process the ComboBox keydown event.  Pressing Ctrl+Delete removes the selected item.
; Parameters:
;	$nKey - IN - Key being pressed.
;	$nShift - IN - Shift state.
; Returns:
;	None.
; ===================================================================
Func _HistoryComboEvent_KeyDown($nKey, $nShift)
	Local $obj = @COM_EventObj
	If $nKey = 0x11 Then	; Ctrl
		$__g_HistoryComboItem = $obj.SelectedItem
	ElseIf $nKey = 0x2E And $nShift = 2 And IsObj($__g_HistoryComboItem) Then	; Delete + Ctrl-down
		_HistoryComboRemoveItem($obj, $__g_HistoryComboItem.Index)
		If $obj.ComboItems.Count Then
			$obj.SelectedItem = $obj.ComboItems.Item(1)
			$__g_HistoryComboItem = $obj.SelectedItem
			_HistoryComboShowDropdown($obj, True)
		Else
			$__g_HistoryComboItem = 0
			_HistoryComboShowDropdown($obj, False)
		EndIf
	EndIf
EndFunc	; _HistoryComboEvent_KeyDown()
#EndRegion _HistoryComboEvent_KeyDown()

#Region _HistoryComboEvent_KeyUp()
; ===================================================================
; _HistoryComboEvent_KeyUp($nKey, $nShift)
;
; Process the ComboBox keyup event.  Pressing Enter will add the current entry to the dropdown.
; Parameters:
;	$nKey - IN - Key being released.
;	$nShift - IN - Shift state.
; Returns:
;	None.
; ===================================================================
Func _HistoryComboEvent_KeyUp($nKey, $nShift)
	Local $obj = @COM_EventObj
	If $nKey = 0x11 Then	; Ctrl
		$__g_HistoryComboItem = 0
	ElseIf $nKey = 0x0D And $nShift = 0 Then	; Enter
		If _HistoryComboIsDropped($obj) Then
			If IsObj($obj.SelectedItem) And $obj.SelectedItem.Text = $obj.Text Then
				_HistoryComboShowDropdown($obj, False)
			Else
				_HistoryComboAddItem($obj, $obj.Text)
			EndIf
		ElseIf $obj.Text Then
			_HistoryComboAddItem($obj, $obj.Text)
			_HistoryComboShowDropdown($obj, True)
		EndIf
	EndIf
EndFunc	; _HistoryComboEvent_KeyUp()
#EndRegion _HistoryComboEvent_KeyUp()

#Region _HistoryComboEvent_DropDown()
; ===================================================================
; _HistoryComboEvent_DropDown()
;
; Refreshes the ComboBox control when the dropdown event occurs.  This ensures no bad painting
;	effects occur.
; Parameters:
;	None.
; Returns:
;	None.
; ===================================================================
Func _HistoryComboEvent_DropDown()
	Local $oCombo = @COM_EventObj
	$oCombo.Refresh()
EndFunc	; _HistoryComboEvent_DropDown()
#EndRegion _HistoryComboEvent_DropDown()

#EndRegion COM Event Callbacks

#Region Private Members

#Region __HistoryComboSendMessage()
; ===================================================================
; __HistoryComboSendMessage(ByRef $vCombo, $nMsg, $wParam = 0, $lParam = 0)
;
; Sends a message to the ComboBox control.
; Parameters:
;	$vCombo - IN - Handle to the HistoryCombo object.
;	$nMsg - IN - Message to send.
;	$wParam - IN/OPTIONAL - WPARAM for the message.
;	$lParam - IN/OPTIONAL - LPARAM for the message.
; Returns:
;	Success - Return value of SendMessage().
;	Failure - Sets @error and returns 0.
; ===================================================================
Func __HistoryComboSendMessage(ByRef $vCombo, $nMsg, $wParam = 0, $lParam = 0)
	Local $oCombo = _HistoryComboGetObj($vCombo)
	If IsObj($oCombo) Then
		Local $aRet = DllCall("user32.dll", "int", "SendMessage", "hwnd", $oCombo.HWND, _
			"int", $nMsg, "int", $wParam, "int", $lParam)
		If Not @error Then Return $aRet[0]
	EndIf
	SetError(1)
	Return 0
EndFunc	; __HistoryComboSendMessage()
#EndRegion __HistoryComboSendMessage()

#EndRegion Private Members																	ConsoleWrite('@@ Debug(484) : Members = ' & Members & @lf & '>Error code: ' & @error & @lf) ;### Debug Console
